@@ -92,7 +92,9 @@ pub fn get_config_string_repo(
 	// gets returned when but it actually works
 	let entry_res = cfg.get_entry(key);
 
-	let Ok(entry) = entry_res else { return Ok(None) };
+	let Ok(entry) = entry_res else {
+		return Ok(None);
+	};
 
 	if entry.has_value() {
 		Ok(entry.value().map(std::string::ToString::to_string))
