@@ -6,7 +6,10 @@ use super::{
 	CommitId, RepoPath,
 };
 use crate::{
-	error::Error, error::Result, hash, sync::repository::repo,
+	error::Error,
+	error::Result,
+	hash,
+	sync::{get_stashes, repository::repo},
 };
 use easy_cast::Conv;
 use git2::{
@@ -223,8 +226,13 @@ pub fn get_diff_commit(
 
 	let repo = repo(repo_path)?;
 	let work_dir = work_dir(&repo)?;
-	let diff =
-		get_commit_diff(repo_path, &repo, id, Some(p), options)?;
+	let diff = get_commit_diff(
+		&repo,
+		id,
+		Some(p),
+		options,
+		Some(&get_stashes(repo_path)?.into_iter().collect()),
+	)?;
 
 	raw_diff_to_file_diff(&diff, work_dir)
 }
