@@ -14,9 +14,7 @@ pub fn get_stashes(repo_path: &RepoPath) -> Result<Vec<CommitId>> {
 	scope_time!("get_stashes");
 
 	let mut repo = repo(repo_path)?;
-
 	let mut list = Vec::new();
-
 	repo.stash_foreach(|_index, _msg, id| {
 		list.push((*id).into());
 		true
@@ -25,15 +23,6 @@ pub fn get_stashes(repo_path: &RepoPath) -> Result<Vec<CommitId>> {
 	Ok(list)
 }
 
-/// checks whether a given commit is a stash commit.
-pub fn is_stash_commit(
-	repo_path: &RepoPath,
-	id: &CommitId,
-) -> Result<bool> {
-	let stashes = get_stashes(repo_path)?;
-	Ok(stashes.contains(id))
-}
-
 ///
 pub fn stash_drop(
 	repo_path: &RepoPath,
