@@ -22,6 +22,7 @@ use ratatui::{
 	Frame,
 };
 use std::{cell::Cell, collections::HashMap, ops::Range};
+use unicode_segmentation::UnicodeSegmentation;
 
 #[derive(PartialEq, Eq)]
 pub enum InputType {
@@ -36,6 +37,7 @@ pub struct TextInputComponent {
 	default_msg: String,
 	msg: String,
 	visible: bool,
+	selected: Option<bool>,
 	show_char_count: bool,
 	theme: SharedTheme,
 	key_config: SharedKeyConfig,
@@ -43,6 +45,7 @@ pub struct TextInputComponent {
 	input_type: InputType,
 	current_area: Cell<Rect>,
 	embed: bool,
+	char_count: usize,
 }
 
 impl TextInputComponent {
@@ -62,10 +65,12 @@ impl TextInputComponent {
 			show_char_count,
 			title: title.to_string(),
 			default_msg: default_msg.to_string(),
+			selected: None,
 			cursor_position: 0,
 			input_type: InputType::Multiline,
 			current_area: Cell::new(Rect::default()),
 			embed: false,
+			char_count: 0,
 		}
 	}
 
@@ -80,6 +85,7 @@ impl TextInputComponent {
 	/// Clear the `msg`.
 	pub fn clear(&mut self) {
 		self.msg.clear();
+		self.update_count();
 		self.cursor_position = 0;
 	}
 
@@ -98,6 +104,11 @@ impl TextInputComponent {
 		self.embed = true;
 	}
 
+	///
+	pub fn enabled(&mut self, enable: bool) {
+		self.selected = Some(enable);
+	}
+
 	/// Move the cursor right one char.
 	fn incr_cursor(&mut self) {
 		if let Some(pos) = self.next_char_position() {
@@ -203,6 +214,7 @@ impl TextInputComponent {
 		if self.cursor_position > 0 {
 			self.decr_cursor();
 			self.msg.remove(self.cursor_position);
+			self.update_count();
 		}
 	}
 
@@ -210,6 +222,7 @@ impl TextInputComponent {
 	pub fn set_text(&mut self, msg: String) {
 		self.msg = msg;
 		self.cursor_position = 0;
+		self.update_count();
 	}
 
 	/// Set the `title`.
@@ -223,7 +236,8 @@ impl TextInputComponent {
 	}
 
 	fn get_draw_text(&self) -> Text {
-		let style = self.theme.text(true, false);
+		let style =
+			self.theme.text(self.selected.unwrap_or(true), false);
 
 		let mut txt = Text::default();
 		// The portion of the text before the cursor is added
@@ -303,10 +317,12 @@ impl TextInputComponent {
 	}
 
 	fn draw_char_count<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
-		let count = self.msg.len();
-		if count > 0 {
-			let w = Paragraph::new(format!("[{count} chars]"))
-				.alignment(Alignment::Right);
+		if self.char_count > 0 {
+			let w = Paragraph::new(format!(
+				"[{} chars]",
+				self.char_count
+			))
+			.alignment(Alignment::Right);
 
 			let mut rect = {
 				let mut rect = r;
@@ -323,6 +339,10 @@ impl TextInputComponent {
 			f.render_widget(w, rect);
 		}
 	}
+
+	fn update_count(&mut self) {
+		self.char_count = self.msg.graphemes(true).count();
+	}
 }
 
 // merges last line of `txt` with first of `append` so we do not generate unneeded newlines
@@ -355,7 +375,10 @@ impl DrawableComponent for TextInputComponent {
 			let txt = if self.msg.is_empty() {
 				Text::styled(
 					self.default_msg.as_str(),
-					self.theme.text(false, false),
+					self.theme.text(
+						self.selected.unwrap_or_default(),
+						false,
+					),
 				)
 			} else {
 				self.get_draw_text()
@@ -432,6 +455,7 @@ impl Component for TextInputComponent {
 				match e.code {
 					KeyCode::Char(c) if !is_ctrl => {
 						self.msg.insert(self.cursor_position, c);
+						self.update_count();
 						self.incr_cursor();
 						return Ok(EventState::Consumed);
 					}
@@ -441,6 +465,7 @@ impl Component for TextInputComponent {
 								self.cursor_position..pos,
 								"",
 							);
+							self.update_count();
 						}
 						return Ok(EventState::Consumed);
 					}
@@ -455,6 +480,7 @@ impl Component for TextInputComponent {
 								"",
 							);
 							self.cursor_position = pos;
+							self.update_count();
 						}
 						return Ok(EventState::Consumed);
 					}
@@ -475,6 +501,7 @@ impl Component for TextInputComponent {
 					KeyCode::Delete => {
 						if self.cursor_position < self.msg.len() {
 							self.msg.remove(self.cursor_position);
+							self.update_count();
 						}
 						return Ok(EventState::Consumed);
 					}
