@@ -1,12 +1,14 @@
 use crate::{
 	components::{
 		AppOption, BlameFileOpen, FileRevOpen, FileTreeOpen,
-		InspectCommitOpen,
+		FuzzyFinderTarget, InspectCommitOpen,
 	},
 	tabs::StashingOptions,
 };
 use asyncgit::{
-	sync::{diff::DiffLinePosition, CommitId, TreeFile},
+	sync::{
+		diff::DiffLinePosition, CommitId, LogFilterSearchOptions,
+	},
 	PushType,
 };
 use bitflags::bitflags;
@@ -68,6 +70,14 @@ pub enum StackablePopupOpen {
 	CompareCommits(InspectCommitOpen),
 }
 
+pub enum AppTabs {
+	Status,
+	Log,
+	Files,
+	Stashing,
+	Stashlist,
+}
+
 ///
 pub enum InternalEvent {
 	///
@@ -89,6 +99,8 @@ pub enum InternalEvent {
 	///
 	TabSwitchStatus,
 	///
+	TabSwitch(AppTabs),
+	///
 	SelectCommitInRevlog(CommitId),
 	///
 	TagCommit(CommitId),
@@ -111,13 +123,11 @@ pub enum InternalEvent {
 	///
 	OptionSwitched(AppOption),
 	///
-	OpenFileFinder(Vec<TreeFile>),
+	OpenFuzzyFinder(Vec<String>, FuzzyFinderTarget),
 	///
-	OpenBranchFinder(Vec<String>),
+	OpenLogSearchPopup,
 	///
-	FileFinderChanged(Option<PathBuf>),
-	///
-	BranchFinderChanged(Option<usize>),
+	FuzzyFinderChanged(usize, String, FuzzyFinderTarget),
 	///
 	FetchRemotes,
 	///
@@ -134,6 +144,8 @@ pub enum InternalEvent {
 	OpenResetPopup(CommitId),
 	///
 	RewordCommit(CommitId),
+	///
+	CommitSearch(LogFilterSearchOptions),
 }
 
 /// single threaded simple queue for components to communicate with each other
