@@ -7,27 +7,49 @@ use crate::{
 	},
 	keys::{key_match, SharedKeyConfig},
 	queue::{InternalEvent, Queue, StackablePopupOpen},
-	strings, try_or_popup,
-	ui::style::SharedTheme,
+	strings::{self, order},
+	try_or_popup,
+	ui::style::{SharedTheme, Theme},
 };
 use anyhow::Result;
 use asyncgit::{
 	asyncjob::AsyncSingleJob,
-	sync::{self, CommitId, RepoPathRef},
-	AsyncBranchesJob, AsyncGitNotification, AsyncLog, AsyncTags,
-	CommitFilesParams, FetchStatus,
+	sync::{
+		self, filter_commit_by_search, CommitId, LogFilterSearch,
+		LogFilterSearchOptions, RepoPathRef,
+	},
+	AsyncBranchesJob, AsyncCommitFilterJob, AsyncGitNotification,
+	AsyncLog, AsyncTags, CommitFilesParams, FetchStatus,
+	ProgressPercent,
 };
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
+use indexmap::IndexSet;
 use ratatui::{
 	backend::Backend,
-	layout::{Constraint, Direction, Layout, Rect},
+	layout::{Alignment, Constraint, Direction, Layout, Rect},
+	text::Span,
+	widgets::{Block, Borders, Paragraph},
 	Frame,
 };
-use std::time::Duration;
+use std::{rc::Rc, time::Duration};
 use sync::CommitTags;
 
-const SLICE_SIZE: usize = 1200;
+struct LogSearchResult {
+	options: LogFilterSearchOptions,
+	duration: Duration,
+}
+
+//TODO: deserves its own component
+enum LogSearch {
+	Off,
+	Searching(
+		AsyncSingleJob<AsyncCommitFilterJob>,
+		LogFilterSearchOptions,
+		Option<ProgressPercent>,
+	),
+	Results(LogSearchResult),
+}
 
 ///
 pub struct Revlog {
@@ -35,12 +57,15 @@ pub struct Revlog {
 	commit_details: CommitDetailsComponent,
 	list: CommitList,
 	git_log: AsyncLog,
+	search: LogSearch,
 	git_tags: AsyncTags,
 	git_local_branches: AsyncSingleJob<AsyncBranchesJob>,
 	git_remote_branches: AsyncSingleJob<AsyncBranchesJob>,
 	queue: Queue,
 	visible: bool,
 	key_config: SharedKeyConfig,
+	sender: Sender<AsyncGitNotification>,
+	theme: SharedTheme,
 }
 
 impl Revlog {
@@ -65,7 +90,7 @@ impl Revlog {
 			list: CommitList::new(
 				repo.clone(),
 				&strings::log_title(&key_config),
-				theme,
+				theme.clone(),
 				queue.clone(),
 				key_config.clone(),
 			),
@@ -74,39 +99,41 @@ impl Revlog {
 				sender,
 				None,
 			),
+			search: LogSearch::Off,
 			git_tags: AsyncTags::new(repo.borrow().clone(), sender),
 			git_local_branches: AsyncSingleJob::new(sender.clone()),
 			git_remote_branches: AsyncSingleJob::new(sender.clone()),
 			visible: false,
 			key_config,
+			sender: sender.clone(),
+			theme,
 		}
 	}
 
 	///
 	pub fn any_work_pending(&self) -> bool {
 		self.git_log.is_pending()
+			|| self.is_search_pending()
 			|| self.git_tags.is_pending()
 			|| self.git_local_branches.is_pending()
 			|| self.git_remote_branches.is_pending()
 			|| self.commit_details.any_work_pending()
 	}
 
+	const fn is_search_pending(&self) -> bool {
+		matches!(self.search, LogSearch::Searching(_, _, _))
+	}
+
 	///
 	pub fn update(&mut self) -> Result<()> {
 		if self.is_visible() {
-			let log_changed =
-				self.git_log.fetch()? == FetchStatus::Started;
-
-			self.list.set_count_total(self.git_log.count()?);
-
-			let selection = self.list.selection();
-			let selection_max = self.list.selection_max();
-			if self.list.items().needs_data(selection, selection_max)
-				|| log_changed
-			{
-				self.fetch_commits()?;
+			if self.git_log.fetch()? == FetchStatus::Started {
+				self.list.clear();
 			}
 
+			self.list
+				.refresh_extend_data(self.git_log.extract_items()?);
+
 			self.git_tags.request(Duration::from_secs(3), false)?;
 
 			if self.commit_details.is_visible() {
@@ -132,6 +159,9 @@ impl Revlog {
 			match ev {
 				AsyncGitNotification::CommitFiles
 				| AsyncGitNotification::Log => self.update()?,
+				AsyncGitNotification::CommitFilter => {
+					self.update_search_state();
+				}
 				AsyncGitNotification::Tags => {
 					if let Some(tags) = self.git_tags.last()? {
 						self.list.set_tags(tags);
@@ -170,26 +200,6 @@ impl Revlog {
 		Ok(())
 	}
 
-	fn fetch_commits(&mut self) -> Result<()> {
-		let want_min =
-			self.list.selection().saturating_sub(SLICE_SIZE / 2);
-
-		let commits = sync::get_commits_info(
-			&self.repo.borrow(),
-			&self.git_log.get_slice(want_min, SLICE_SIZE)?,
-			self.list
-				.current_size()
-				.map_or(100u16, |size| size.0)
-				.into(),
-		);
-
-		if let Ok(commits) = commits {
-			self.list.items().set_items(want_min, commits);
-		}
-
-		Ok(())
-	}
-
 	fn selected_commit(&self) -> Option<CommitId> {
 		self.list.selected_entry().map(|e| e.id)
 	}
@@ -205,16 +215,9 @@ impl Revlog {
 		})
 	}
 
+	///
 	pub fn select_commit(&mut self, id: CommitId) -> Result<()> {
-		let position = self.git_log.position(id)?;
-
-		if let Some(position) = position {
-			self.list.select_entry(position);
-
-			Ok(())
-		} else {
-			anyhow::bail!("Could not select commit in revlog. It might not be loaded yet or it might be on a different branch.");
-		}
+		self.list.select_commit(id)
 	}
 
 	fn revert_commit(&self) -> Result<()> {
@@ -236,6 +239,139 @@ impl Revlog {
 			));
 		}
 	}
+
+	pub fn search(&mut self, options: LogFilterSearchOptions) {
+		if !self.can_start_search() {
+			return;
+		}
+
+		if matches!(
+			self.search,
+			LogSearch::Off | LogSearch::Results(_)
+		) {
+			log::info!("start search: {:?}", options);
+
+			let filter = filter_commit_by_search(
+				LogFilterSearch::new(options.clone()),
+			);
+
+			let mut job = AsyncSingleJob::new(self.sender.clone());
+			job.spawn(AsyncCommitFilterJob::new(
+				self.repo.borrow().clone(),
+				self.list.copy_items(),
+				filter,
+			));
+
+			self.search = LogSearch::Searching(job, options, None);
+
+			self.list.set_highlighting(None);
+		}
+	}
+
+	fn update_search_state(&mut self) {
+		match &mut self.search {
+			LogSearch::Off | LogSearch::Results(_) => (),
+			LogSearch::Searching(search, options, progress) => {
+				if search.is_pending() {
+					//update progress
+					*progress = search.progress();
+				} else if let Some(search) = search
+					.take_last()
+					.and_then(|search| search.result())
+				{
+					match search {
+						Ok(search) => {
+							self.list.set_highlighting(Some(
+								Rc::new(
+									search
+										.result
+										.into_iter()
+										.collect::<IndexSet<_>>(),
+								),
+							));
+
+							self.search =
+								LogSearch::Results(LogSearchResult {
+									options: options.clone(),
+									duration: search.duration,
+								});
+						}
+						Err(err) => {
+							self.queue.push(
+								InternalEvent::ShowErrorMsg(format!(
+									"search error: {err}",
+								)),
+							);
+
+							self.search = LogSearch::Off;
+						}
+					}
+				}
+			}
+		}
+	}
+
+	fn is_in_search_mode(&self) -> bool {
+		!matches!(self.search, LogSearch::Off)
+	}
+
+	fn draw_search<B: Backend>(&self, f: &mut Frame<B>, area: Rect) {
+		let (text, title) = match &self.search {
+			LogSearch::Searching(_, options, progress) => (
+				format!("'{}'", options.search_pattern.clone()),
+				format!(
+					"({}%)",
+					progress
+						.map(|progress| progress.progress)
+						.unwrap_or_default()
+				),
+			),
+			LogSearch::Results(results) => {
+				let info = self.list.highlighted_selection_info();
+
+				(
+					format!(
+						"'{}' (duration: {:?})",
+						results.options.search_pattern.clone(),
+						results.duration,
+					),
+					format!(
+						"({}/{})",
+						(info.0 + 1).min(info.1),
+						info.1
+					),
+				)
+			}
+			LogSearch::Off => (String::new(), String::new()),
+		};
+
+		f.render_widget(
+			Paragraph::new(text)
+				.block(
+					Block::default()
+						.title(Span::styled(
+							format!(
+								"{} {}",
+								strings::POPUP_TITLE_LOG_SEARCH,
+								title
+							),
+							self.theme.title(true),
+						))
+						.borders(Borders::ALL)
+						.border_style(Theme::attention_block()),
+				)
+				.alignment(Alignment::Left),
+			area,
+		);
+	}
+
+	fn can_leave_search(&self) -> bool {
+		self.is_in_search_mode() && !self.is_search_pending()
+	}
+
+	fn can_start_search(&self) -> bool {
+		!self.git_log.is_pending()
+	}
 }
 
 impl DrawableComponent for Revlog {
@@ -244,6 +380,18 @@ impl DrawableComponent for Revlog {
 		f: &mut Frame<B>,
 		area: Rect,
 	) -> Result<()> {
+		let area = if self.is_in_search_mode() {
+			Layout::default()
+				.direction(Direction::Vertical)
+				.constraints(
+					[Constraint::Min(1), Constraint::Length(3)]
+						.as_ref(),
+				)
+				.split(area)
+		} else {
+			Rc::new([area])
+		};
+
 		let chunks = Layout::default()
 			.direction(Direction::Horizontal)
 			.constraints(
@@ -253,13 +401,17 @@ impl DrawableComponent for Revlog {
 				]
 				.as_ref(),
 			)
-			.split(area);
+			.split(area[0]);
 
 		if self.commit_details.is_visible() {
 			self.list.draw(f, chunks[0])?;
 			self.commit_details.draw(f, chunks[1])?;
 		} else {
-			self.list.draw(f, area)?;
+			self.list.draw(f, area[0])?;
+		}
+
+		if self.is_in_search_mode() {
+			self.draw_search(f, area[1]);
 		}
 
 		Ok(())
@@ -281,6 +433,15 @@ impl Component for Revlog {
 					self.commit_details.toggle_visible()?;
 					self.update()?;
 					return Ok(EventState::Consumed);
+				} else if key_match(
+					k,
+					self.key_config.keys.exit_popup,
+				) {
+					if self.can_leave_search() {
+						self.search = LogSearch::Off;
+						self.list.set_highlighting(None);
+						return Ok(EventState::Consumed);
+					}
 				} else if key_match(k, self.key_config.keys.copy) {
 					try_or_popup!(
 						self,
@@ -373,6 +534,12 @@ impl Component for Revlog {
 							Ok(EventState::Consumed)
 						},
 					);
+				} else if key_match(k, self.key_config.keys.log_find)
+					&& self.can_start_search()
+				{
+					self.queue
+						.push(InternalEvent::OpenLogSearchPopup);
+					return Ok(EventState::Consumed);
 				} else if key_match(
 					k,
 					self.key_config.keys.compare_commits,
@@ -418,6 +585,16 @@ impl Component for Revlog {
 			self.list.commands(out, force_all);
 		}
 
+		out.push(
+			CommandInfo::new(
+				strings::commands::log_close_search(&self.key_config),
+				true,
+				(self.visible && self.can_leave_search())
+					|| force_all,
+			)
+			.order(order::PRIORITY),
+		);
+
 		out.push(CommandInfo::new(
 			strings::commands::log_details_toggle(&self.key_config),
 			true,
@@ -505,6 +682,11 @@ impl Component for Revlog {
 			self.selected_commit().is_some(),
 			self.visible || force_all,
 		));
+		out.push(CommandInfo::new(
+			strings::commands::log_find_commit(&self.key_config),
+			self.can_start_search(),
+			self.visible || force_all,
+		));
 
 		visibility_blocking(self)
 	}
@@ -520,7 +702,6 @@ impl Component for Revlog {
 
 	fn show(&mut self) -> Result<()> {
 		self.visible = true;
-		self.list.clear();
 
 		self.git_local_branches.spawn(AsyncBranchesJob::new(
 			self.repo.borrow().clone(),
